@@ -25,9 +25,6 @@ Asynchronous job scheduler, for concurrent execution with minimalistic
 dependency guarantees.
 """
 
-import sys
-import gettext
-
 import duplicity
 from duplicity import log
 from duplicity.dup_threading import require_threading
@@ -35,9 +32,10 @@ from duplicity.dup_threading import interruptably_wait
 from duplicity.dup_threading import async_split
 from duplicity.dup_threading import with_lock
 
-thread    = duplicity.dup_threading.thread_module()
+thread = duplicity.dup_threading.thread_module()
 threading = duplicity.dup_threading.threading_module()
 
+
 class AsyncScheduler:
     """
     Easy-to-use scheduler of function calls to be executed
@@ -67,18 +65,18 @@ class AsyncScheduler:
         """
         log.Info("%s: %s" % (self.__class__.__name__,
                              _("instantiating at concurrency %d") %
-                               (concurrency)))
+                             (concurrency)))
         assert concurrency >= 0, "%s concurrency level must be >= 0" % (self.__class__.__name__,)
 
-        self.__failed        = False        # has at least one task failed so far?
-        self.__failed_waiter = None         # when __failed, the waiter of the first task that failed
-        self.__concurrency   = concurrency
-        self.__worker_count  = 0            # number of active workers
-        self.__waiter_count  = 0            # number of threads waiting to submit work
-        self.__barrier       = False        # barrier currently in effect?
-        self.__cv            = threading.Condition() # for simplicity, we use a single cv with its lock
-                                                     # for everything, even if the resulting notifyAll():s
-                                                     # are not technically efficient.
+        self.__failed = False  # has at least one task failed so far?
+        self.__failed_waiter = None  # when __failed, the waiter of the first task that failed
+        self.__concurrency = concurrency
+        self.__worker_count = 0  # number of active workers
+        self.__waiter_count = 0  # number of threads waiting to submit work
+        self.__barrier = False  # barrier currently in effect?
+        self.__cv = threading.Condition()  # for simplicity, we use a single cv with its lock
+#                                                    # for everything, even if the resulting notifyAll():s
+#                                                    # are not technically efficient.
 
         if concurrency > 0:
             require_threading("concurrency > 0 (%d)" % (concurrency,))
@@ -168,7 +166,6 @@ class AsyncScheduler:
         with_lock(self.__cv, _wait)
 
     def __run_synchronously(self, fn, params):
-        success = False
 
         # When running synchronously, we immediately leak any exception raised
         # for immediate failure reporting to calling code.
@@ -197,7 +194,7 @@ class AsyncScheduler:
                                      "this is a bug" % (self.__class__.__name__,))
 
         def wait_for_and_register_launch():
-            check_pending_failure()    # raise on fail
+            check_pending_failure()  # raise on fail
             while self.__worker_count >= self.__concurrency or self.__barrier:
                 if self.__worker_count == 0:
                     assert self.__barrier, "barrier should be in effect"
@@ -208,12 +205,12 @@ class AsyncScheduler:
                     self.__cv.wait()
                     self.__waiter_count -= 1
 
-                check_pending_failure() # raise on fail
+                check_pending_failure()  # raise on fail
 
             self.__worker_count += 1
             log.Debug("%s: %s" % (self.__class__.__name__,
                                   _("active workers = %d") % (self.__worker_count,)))
-    
+
         # simply wait for an OK condition to start, then launch our worker. the worker
         # never waits on us, we just wait for them.
         with_lock(self.__cv, wait_for_and_register_launch)
