@@ -22,18 +22,20 @@
 # Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 
 import base64
-import httplib, os
+import httplib
+import os
 import re
 import urllib
 import urllib2
+import urlparse
 import xml.dom.minidom
 
 import duplicity.backend
 from duplicity import globals
 from duplicity import log
-from duplicity.errors import * #@UnusedWildImport
-from duplicity import urlparse_2_5 as urlparser
-from duplicity.backend import retry_fatal
+from duplicity import util
+from duplicity.errors import BackendException, FatalBackendException
+
 
 class CustomMethodRequest(urllib2.Request):
     """
@@ -48,36 +50,33 @@ class CustomMethodRequest(urllib2.Request):
     def get_method(self):
         return self.method
 
+
 class VerifiedHTTPSConnection(httplib.HTTPSConnection):
         def __init__(self, *args, **kwargs):
             try:
                 global socket, ssl
-                import socket, ssl
+                import socket
+                import ssl
             except ImportError:
-                raise FatalBackendError("Missing socket or ssl libraries.")
+                raise FatalBackendException(_("Missing socket or ssl python modules."))
 
             httplib.HTTPSConnection.__init__(self, *args, **kwargs)
 
             self.cacert_file = globals.ssl_cacert_file
-            cacert_candidates = [ "~/.duplicity/cacert.pem", \
-                             "~/duplicity_cacert.pem", \
-                             "/etc/duplicity/cacert.pem" ]
-            #
+            self.cacert_candidates = ["~/.duplicity/cacert.pem",
+                                      "~/duplicity_cacert.pem",
+                                      "/etc/duplicity/cacert.pem"]
+            # if no cacert file was given search default locations
             if not self.cacert_file:
-                for path in cacert_candidates :
+                for path in self.cacert_candidates:
                     path = os.path.expanduser(path)
                     if (os.path.isfile(path)):
                         self.cacert_file = path
                         break
-            # still no cacert file, inform user
-            if not self.cacert_file:
-                raise FatalBackendError("""For certificate verification a cacert database file is needed in one of these locations: %s
-Hints:
-  Consult the man page, chapter 'SSL Certificate Verification'.
-  Consider using the options --ssl-cacert-file, --ssl-no-check-certificate .""" % ", ".join(cacert_candidates) )
+
             # check if file is accessible (libssl errors are not very detailed)
-            if not os.access(self.cacert_file, os.R_OK):
-                raise FatalBackendError("Cacert database file '%s' is not readable." % cacert_file)
+            if self.cacert_file and not os.access(self.cacert_file, os.R_OK):
+                raise FatalBackendException(_("Cacert database file '%s' is not readable.") % self.cacert_file)
 
         def connect(self):
             # create new socket
@@ -85,20 +84,36 @@ Hints:
                                             self.timeout)
             if self._tunnel_host:
                 self.sock = sock
-                self._tunnel()
+                self.tunnel()
 
-            # wrap the socket in ssl using verification
-            self.sock = ssl.wrap_socket(sock,
-                                        cert_reqs=ssl.CERT_REQUIRED,
-                                        ca_certs=self.cacert_file,
-                                        )
+            # python 2.7.9+ supports default system certs now
+            if "create_default_context" in dir(ssl):
+                context = ssl.create_default_context(ssl.Purpose.SERVER_AUTH, cafile=self.cacert_file, capath=globals.ssl_cacert_path)
+                self.sock = context.wrap_socket(sock, server_hostname=self.host)
+            # the legacy way needing a cert file
+            else:
+                if globals.ssl_cacert_path:
+                    raise FatalBackendException(_("Option '--ssl-cacert-path' is not supported with python 2.7.8 and below."))
 
-        def request(self, *args, **kwargs):
+                if not self.cacert_file:
+                    raise FatalBackendException(_("""\
+For certificate verification with python 2.7.8 or earlier a cacert database file is needed in one of these locations: %s
+Hints:
+  Consult the man page, chapter 'SSL Certificate Verification'.
+  Consider using the options --ssl-cacert-file, --ssl-no-check-certificate .""") % ", ".join(self.cacert_candidates))
+
+                # wrap the socket in ssl using verification
+                self.sock = ssl.wrap_socket(sock,
+                                            cert_reqs=ssl.CERT_REQUIRED,
+                                            ca_certs=self.cacert_file,
+                                            )
+
+        def request(self, *args, **kwargs):  # pylint: disable=method-hidden
             try:
                 return httplib.HTTPSConnection.request(self, *args, **kwargs)
-            except ssl.SSLError, e:
+            except ssl.SSLError as e:
                 # encapsulate ssl errors
-                raise BackendException("SSL failed: %s" % str(e),log.ErrorCode.backend_error)
+                raise BackendException("SSL failed: %s" % util.uexc(e), log.ErrorCode.backend_error)
 
 
 class WebDAVBackend(duplicity.backend.Backend):
@@ -108,13 +123,9 @@ class WebDAVBackend(duplicity.backend.Backend):
     """
 
     """
-    for better compatibility we send an empty listbody as described in
-    http://www.ietf.org/rfc/rfc4918.txt
-    "  A client may choose not to submit a request body.  An empty PROPFIND
-       request body MUST be treated as if it were an 'allprop' request.  "
-    it was retired because e.g. box.net didn't support <D:allprop/>
+    Request just the names.
     """
-    listbody =""
+    listbody = '<?xml version="1.0"?><D:propfind xmlns:D="DAV:"><D:prop><D:resourcetype/></D:prop></D:propfind>'
 
     """Connect to remote store using WebDAV Protocol"""
     def __init__(self, parsed_url):
@@ -126,98 +137,102 @@ class WebDAVBackend(duplicity.backend.Backend):
 
         self.username = parsed_url.username
         self.password = self.get_password()
-        self.directory = self._sanitize_path(parsed_url.path)
+        self.directory = self.sanitize_path(parsed_url.path)
 
-        log.Info("Using WebDAV protocol %s" % (globals.webdav_proto,))
-        log.Info("Using WebDAV host %s port %s" % (parsed_url.hostname, parsed_url.port))
-        log.Info("Using WebDAV directory %s" % (self.directory,))
+        log.Info(_("Using WebDAV protocol %s") % (globals.webdav_proto,))
+        log.Info(_("Using WebDAV host %s port %s") % (parsed_url.hostname, parsed_url.port))
+        log.Info(_("Using WebDAV directory %s") % (self.directory,))
 
         self.conn = None
 
-    def _sanitize_path(self,path):
+    def sanitize_path(self, path):
         if path:
             foldpath = re.compile('/+')
-            return foldpath.sub('/', path + '/' )
+            return foldpath.sub('/', path + '/')
         else:
             return '/'
 
-    def _getText(self,nodelist):
+    def getText(self, nodelist):
         rc = ""
         for node in nodelist:
             if node.nodeType == node.TEXT_NODE:
                 rc = rc + node.data
         return rc
 
-    def _connect(self, forced=False):
+    def _retry_cleanup(self):
+        self.connect(forced=True)
+
+    def connect(self, forced=False):
         """
         Connect or re-connect to the server, updates self.conn
         # reconnect on errors as a precaution, there are errors e.g.
         # "[Errno 32] Broken pipe" or SSl errors that render the connection unusable
         """
-        if self.retry_count<=1 and self.conn \
-            and self.conn.host == self.parsed_url.hostname: return
+        if not forced and self.conn \
+                and self.conn.host == self.parsed_url.hostname:
+            return
 
-        log.Info("WebDAV create connection on '%s' (retry %s) " % (self.parsed_url.hostname,self.retry_count) )
-        if self.conn: self.conn.close()
+        log.Info(_("WebDAV create connection on '%s'") % (self.parsed_url.hostname))
+        self._close()
         # http schemes needed for redirect urls from servers
-        if self.parsed_url.scheme in ['webdav','http']:
+        if self.parsed_url.scheme in ['webdav', 'http']:
             self.conn = httplib.HTTPConnection(self.parsed_url.hostname, self.parsed_url.port)
-        elif self.parsed_url.scheme in ['webdavs','https']:
+        elif self.parsed_url.scheme in ['webdavs', 'https']:
             if globals.ssl_no_check_certificate:
                 self.conn = httplib.HTTPSConnection(self.parsed_url.hostname, self.parsed_url.port)
             else:
                 self.conn = VerifiedHTTPSConnection(self.parsed_url.hostname, self.parsed_url.port)
         else:
-            raise FatalBackendError("WebDAV Unknown URI scheme: %s" % (self.parsed_url.scheme))
+            raise FatalBackendException(_("WebDAV Unknown URI scheme: %s") % (self.parsed_url.scheme))
 
-    def close(self):
-        self.conn.close()
+    def _close(self):
+        if self.conn:
+            self.conn.close()
 
     def request(self, method, path, data=None, redirected=0):
         """
         Wraps the connection.request method to retry once if authentication is
         required
         """
-        self._connect()
+        self._close()  # or we get previous request's data or exception
+        self.connect()
 
-        quoted_path = urllib.quote(path,"/:~")
+        quoted_path = urllib.quote(path, "/:~")
 
         if self.digest_challenge is not None:
             self.headers['Authorization'] = self.get_digest_authorization(path)
 
-        log.Info("WebDAV %s %s request with headers: %s " % (method,quoted_path,self.headers))
-        log.Info("WebDAV data length: %s " % len(str(data)) )
+        log.Info(_("WebDAV %s %s request with headers: %s ") % (method, quoted_path, self.headers))
+        log.Info(_("WebDAV data length: %s ") % len(str(data)))
         self.conn.request(method, quoted_path, data, self.headers)
         response = self.conn.getresponse()
-        log.Info("WebDAV response status %s with reason '%s'." % (response.status,response.reason))
+        log.Info(_("WebDAV response status %s with reason '%s'.") % (response.status, response.reason))
         # resolve redirects and reset url on listing requests (they usually come before everything else)
-        if response.status in [301,302] and method == 'PROPFIND':
-            redirect_url = response.getheader('location',None)
+        if response.status in [301, 302] and method == 'PROPFIND':
+            redirect_url = response.getheader('location', None)
             response.close()
             if redirect_url:
-                log.Notice("WebDAV redirect to: %s " % urllib.unquote(redirect_url) )
+                log.Notice(_("WebDAV redirect to: %s ") % urllib.unquote(redirect_url))
                 if redirected > 10:
-                    raise FatalBackendError("WebDAV redirected 10 times. Giving up.")
+                    raise FatalBackendException(_("WebDAV redirected 10 times. Giving up."))
                 self.parsed_url = duplicity.backend.ParsedUrl(redirect_url)
-                self.directory = self._sanitize_path(self.parsed_url.path)
-                return self.request(method,self.directory,data,redirected+1)
+                self.directory = self.sanitize_path(self.parsed_url.path)
+                return self.request(method, self.directory, data, redirected + 1)
             else:
-                raise FatalBackendError("WebDAV missing location header in redirect response.")
+                raise FatalBackendException(_("WebDAV missing location header in redirect response."))
         elif response.status == 401:
             response.read()
             response.close()
             self.headers['Authorization'] = self.get_authorization(response, quoted_path)
-            log.Info("WebDAV retry request with authentification headers.")
-            log.Info("WebDAV %s %s request2 with headers: %s " % (method,quoted_path,self.headers))
-            log.Info("WebDAV data length: %s " % len(str(data)) )
+            log.Info(_("WebDAV retry request with authentification headers."))
+            log.Info(_("WebDAV %s %s request2 with headers: %s ") % (method, quoted_path, self.headers))
+            log.Info(_("WebDAV data length: %s ") % len(str(data)))
             self.conn.request(method, quoted_path, data, self.headers)
             response = self.conn.getresponse()
-            log.Info("WebDAV response2 status %s with reason '%s'." % (response.status,response.reason))
+            log.Info(_("WebDAV response2 status %s with reason '%s'.") % (response.status, response.reason))
 
         return response
 
-
-
     def get_authorization(self, response, path):
         """
         Fetches the auth header based on the requested method (basic or digest)
@@ -227,7 +242,18 @@ class WebDAVBackend(duplicity.backend.Backend):
             token, challenge = auth_hdr.split(' ', 1)
         except ValueError:
             return None
-        if token.lower() == 'basic':
+        if token.split(',')[0].lower() == 'negotiate':
+            try:
+                return self.get_kerberos_authorization()
+            except ImportError:
+                log.warn(_("python-kerberos needed to use kerberos \
+                          authorization, falling back to basic auth."))
+                return self.get_basic_authorization()
+            except Exception as e:
+                log.warn(_("Kerberos authorization failed: %s.\
+                          Falling back to basic auth.") % e)
+                return self.get_basic_authorization()
+        elif token.lower() == 'basic':
             return self.get_basic_authorization()
         else:
             self.digest_challenge = self.parse_digest_challenge(challenge)
@@ -236,6 +262,13 @@ class WebDAVBackend(duplicity.backend.Backend):
     def parse_digest_challenge(self, challenge_string):
         return urllib2.parse_keqv_list(urllib2.parse_http_list(challenge_string))
 
+    def get_kerberos_authorization(self):
+        import kerberos
+        _, ctx = kerberos.authGSSClientInit("HTTP@%s" % self.conn.host)
+        kerberos.authGSSClientStep(ctx, "")
+        tgt = kerberos.authGSSClientResponse(ctx)
+        return 'Negotiate %s' % tgt
+
     def get_basic_authorization(self):
         """
         Returns the basic auth header
@@ -262,10 +295,7 @@ class WebDAVBackend(duplicity.backend.Backend):
         auth_string = self.digest_auth_handler.get_authorization(dummy_req, self.digest_challenge)
         return 'Digest %s' % auth_string
 
-    @retry_fatal
     def _list(self):
-        """List files in directory"""
-        log.Info("Listing directory %s on WebDAV server" % (self.directory,))
         response = None
         try:
             self.headers['Depth'] = "1"
@@ -273,7 +303,7 @@ class WebDAVBackend(duplicity.backend.Backend):
             del self.headers['Depth']
             # if the target collection does not exist, create it.
             if response.status == 404:
-                response.close() # otherwise next request fails with ResponseNotReady
+                response.close()  # otherwise next request fails with ResponseNotReady
                 self.makedir()
                 # just created an empty folder, so return empty
                 return []
@@ -284,20 +314,21 @@ class WebDAVBackend(duplicity.backend.Backend):
                 status = response.status
                 reason = response.reason
                 response.close()
-                raise BackendException("Bad status code %s reason %s." % (status,reason))
+                raise BackendException("Bad status code %s reason %s." % (status, reason))
 
             log.Debug("%s" % (document,))
             dom = xml.dom.minidom.parseString(document)
             result = []
             for href in dom.getElementsByTagName('d:href') + dom.getElementsByTagName('D:href'):
-                filename = self.__taste_href(href)
+                filename = self.taste_href(href)
                 if filename:
                     result.append(filename)
-            if response: response.close()
             return result
-        except Exception, e:
-            if response: response.close()
+        except Exception as e:
             raise e
+        finally:
+            if response:
+                response.close()
 
     def makedir(self):
         """Make (nested) directories on the server."""
@@ -305,11 +336,10 @@ class WebDAVBackend(duplicity.backend.Backend):
         # url causes directory to start with /, but it might be given
         # with or without trailing / (which is required)
         if dirs[-1] == '':
-            dirs=dirs[0:-1]
-        for i in range(1,len(dirs)):
-            d="/".join(dirs[0:i+1])+"/"
+            dirs = dirs[0:-1]
+        for i in range(1, len(dirs)):
+            d = "/".join(dirs[0:i + 1]) + "/"
 
-            self.close() # or we get previous request's data or exception
             self.headers['Depth'] = "1"
             response = self.request("PROPFIND", d)
             del self.headers['Depth']
@@ -317,26 +347,24 @@ class WebDAVBackend(duplicity.backend.Backend):
             log.Info("Checking existence dir %s: %d" % (d, response.status))
 
             if response.status == 404:
-                log.Info("Creating missing directory %s" % d)
-                self.close() # or we get previous request's data or exception
+                log.Info(_("Creating missing directory %s") % d)
 
                 res = self.request("MKCOL", d)
                 if res.status != 201:
-                    raise BackendException("WebDAV MKCOL %s failed: %s %s" % (d,res.status,res.reason))
-                self.close()
+                    raise BackendException(_("WebDAV MKCOL %s failed: %s %s") % (d, res.status, res.reason))
 
-    def __taste_href(self, href):
+    def taste_href(self, href):
         """
         Internal helper to taste the given href node and, if
         it is a duplicity file, collect it as a result file.
 
         @return: A matching filename, or None if the href did not match.
         """
-        raw_filename = self._getText(href.childNodes).strip()
-        parsed_url = urlparser.urlparse(urllib.unquote(raw_filename))
+        raw_filename = self.getText(href.childNodes).strip()
+        parsed_url = urlparse.urlparse(urllib.unquote(raw_filename))
         filename = parsed_url.path
-        log.Debug("webdav path decoding and translation: "
-                  "%s -> %s" % (raw_filename, filename))
+        log.Debug(_("WebDAV path decoding and translation: "
+                  "%s -> %s") % (raw_filename, filename))
 
         # at least one WebDAV server returns files in the form
         # of full URL:s. this may or may not be
@@ -344,7 +372,7 @@ class WebDAVBackend(duplicity.backend.Backend):
         # feel we want to bail out if the hostname
         # does not match until someone has looked into
         # what the WebDAV protocol mandages.
-        if not parsed_url.hostname is None \
+        if parsed_url.hostname is not None \
            and not (parsed_url.hostname == self.parsed_url.hostname):
             m = "Received filename was in the form of a "\
                 "full url, but the hostname (%s) did "\
@@ -357,45 +385,37 @@ class WebDAVBackend(duplicity.backend.Backend):
             raise BackendException(m)
 
         if filename.startswith(self.directory):
-            filename = filename.replace(self.directory,'',1)
+            filename = filename.replace(self.directory, '', 1)
             return filename
         else:
             return None
 
-    @retry_fatal
-    def get(self, remote_filename, local_path):
-        """Get remote filename, saving it to local_path"""
+    def _get(self, remote_filename, local_path):
         url = self.directory + remote_filename
-        log.Info("Retrieving %s from WebDAV server" % (url ,))
         response = None
         try:
             target_file = local_path.open("wb")
             response = self.request("GET", url)
             if response.status == 200:
-                #data=response.read()
+                # data=response.read()
                 target_file.write(response.read())
-                #import hashlib
-                #log.Info("WebDAV GOT %s bytes with md5=%s" % (len(data),hashlib.md5(data).hexdigest()) )
+                # import hashlib
+                # log.Info("WebDAV GOT %s bytes with md5=%s" % (len(data),hashlib.md5(data).hexdigest()) )
                 assert not target_file.close()
-                local_path.setdata()
                 response.close()
             else:
                 status = response.status
                 reason = response.reason
                 response.close()
-                raise BackendException("Bad status code %s reason %s." % (status,reason))
-            if response: response.close()
-        except Exception, e:
-            if response: response.close()
+                raise BackendException(_("WebDAV GET Bad status code %s reason %s.") % (status, reason))
+        except Exception as e:
             raise e
+        finally:
+            if response:
+                response.close()
 
-    @retry_fatal
-    def put(self, source_path, remote_filename = None):
-        """Transfer source_path to remote_filename"""
-        if not remote_filename:
-            remote_filename = source_path.get_filename()
+    def _put(self, source_path, remote_filename):
         url = self.directory + remote_filename
-        log.Info("Saving %s on WebDAV server" % (url ,))
         response = None
         try:
             source_file = source_path.open("rb")
@@ -408,33 +428,34 @@ class WebDAVBackend(duplicity.backend.Backend):
                 status = response.status
                 reason = response.reason
                 response.close()
-                raise BackendException("Bad status code %s reason %s." % (status,reason))
-            if response: response.close()
-        except Exception, e:
-            if response: response.close()
+                raise BackendException(_("WebDAV PUT Bad status code %s reason %s.") % (status, reason))
+        except Exception as e:
             raise e
+        finally:
+            if response:
+                response.close()
 
-    @retry_fatal
-    def delete(self, filename_list):
-        """Delete files in filename_list"""
-        for filename in filename_list:
-            url = self.directory + filename
-            log.Info("Deleting %s from WebDAV server" % (url ,))
-            response = None
-            try:
-                response = self.request("DELETE", url)
-                if response.status in [200, 204]:
-                    response.read()
-                    response.close()
-                else:
-                    status = response.status
-                    reason = response.reason
-                    response.close()
-                    raise BackendException("Bad status code %s reason %s." % (status,reason))
-                if response: response.close()
-            except Exception, e:
-                if response: response.close()
-                raise e
+    def _delete(self, filename):
+        url = self.directory + filename
+        response = None
+        try:
+            response = self.request("DELETE", url)
+            if response.status in [200, 204]:
+                response.read()
+                response.close()
+            else:
+                status = response.status
+                reason = response.reason
+                response.close()
+                raise BackendException(_("WebDAV DEL Bad status code %s reason %s.") % (status, reason))
+        except Exception as e:
+            raise e
+        finally:
+            if response:
+                response.close()
 
+duplicity.backend.register_backend("http", WebDAVBackend)
+duplicity.backend.register_backend("https", WebDAVBackend)
 duplicity.backend.register_backend("webdav", WebDAVBackend)
 duplicity.backend.register_backend("webdavs", WebDAVBackend)
+duplicity.backend.uses_netloc.extend(['http', 'https', 'webdav', 'webdavs'])
