@@ -21,20 +21,27 @@
 
 """Classes and functions on collections of backup volumes"""
 
+from future_builtins import filter, map
+
 import types
 import gettext
 
+
 from duplicity import log
 from duplicity import file_naming
 from duplicity import path
+from duplicity import util
 from duplicity import dup_time
 from duplicity import globals
 from duplicity import manifest
+from duplicity import util
 from duplicity.gpg import GPGError
 
+
 class CollectionsError(Exception):
     pass
 
+
 class BackupSet:
     """
     Backup set - the backup information produced by one session
@@ -44,15 +51,16 @@ class BackupSet:
         Initialize new backup set, only backend is required at first
         """
         self.backend = backend
-        self.info_set = False                       # true if fields are set
-        self.volume_name_dict = {}                  # dict from volume number to filename
-        self.remote_manifest_name = None            # full name of remote manifest
-        self.local_manifest_path = None             # full path to local manifest
-        self.time = None                            # will be set if is full backup set
-        self.start_time = None                      # will be set if inc
-        self.end_time = None                        # will be set if inc
-        self.partial = False                        # true if a partial backup
-        self.encrypted = False                      # true if an encrypted backup
+        self.info_set = False  # true if fields are set
+        self.volume_name_dict = {}  # dict from volume number to filename
+        self.remote_manifest_name = None  # full name of remote manifest
+        self.local_manifest_path = None  # full path to local manifest
+        self.time = None  # will be set if is full backup set
+        self.start_time = None  # will be set if inc
+        self.end_time = None  # will be set if inc
+        self.partial = False  # true if a partial backup
+        self.encrypted = False  # true if an encrypted backup
+        self.files_changed = []
 
     def is_complete(self):
         """
@@ -83,7 +91,7 @@ class BackupSet:
             if pr.time != self.time:
                 return False
             if (pr.start_time != self.start_time or
-                pr.end_time != self.end_time):
+                    pr.end_time != self.end_time):
                 return False
             if bool(pr.encrypted) != bool(self.encrypted):
                 if self.partial and pr.encrypted:
@@ -93,8 +101,8 @@ class BackupSet:
             self.set_manifest(filename)
         else:
             assert pr.volume_number is not None
-            assert not self.volume_name_dict.has_key(pr.volume_number), \
-                   (self.volume_name_dict, filename)
+            assert pr.volume_number not in self.volume_name_dict, \
+                (self.volume_name_dict, filename)
             self.volume_name_dict[pr.volume_number] = filename
 
         return True
@@ -116,6 +124,10 @@ class BackupSet:
         self.encrypted = bool(pr.encrypted)
         self.info_set = True
 
+    def set_files_changed(self):
+        mf = self.get_manifest()
+        self.files_changed = mf.get_files_changed()
+
     def set_manifest(self, remote_filename):
         """
         Add local and remote manifest filenames to backup set
@@ -127,12 +139,14 @@ class BackupSet:
         for local_filename in globals.archive_dir.listdir():
             pr = file_naming.parse(local_filename)
             if (pr and pr.manifest
-                and pr.type == self.type
-                and pr.time == self.time
-                and pr.start_time == self.start_time
-                and pr.end_time == self.end_time):
+                    and pr.type == self.type
+                    and pr.time == self.time
+                    and pr.start_time == self.start_time
+                    and pr.end_time == self.end_time):
                 self.local_manifest_path = \
-                              globals.archive_dir.append(local_filename)
+                    globals.archive_dir.append(local_filename)
+
+                self.set_files_changed()
                 break
 
     def delete(self):
@@ -144,21 +158,22 @@ class BackupSet:
         try:
             self.backend.delete(rfn)
         except Exception:
-            log.Debug("BackupSet.delete: missing %s" % rfn)
+            log.Debug(_("BackupSet.delete: missing %s") % [util.ufn(f) for f in rfn])
             pass
         for lfn in globals.archive_dir.listdir():
             pr = file_naming.parse(lfn)
             if (pr
-                and pr.time == self.time
-                and pr.start_time == self.start_time
-                and pr.end_time == self.end_time):
+                    and pr.time == self.time
+                    and pr.start_time == self.start_time
+                    and pr.end_time == self.end_time):
                 try:
                     globals.archive_dir.append(lfn).delete()
                 except Exception:
-                    log.Debug("BackupSet.delete: missing %s" % lfn)
+                    log.Debug(_("BackupSet.delete: missing %s") % [util.ufn(f) for f in lfn])
                     pass
+        util.release_lockfile()
 
-    def __str__(self):
+    def __unicode__(self):
         """
         For now just list files in set
         """
@@ -166,7 +181,7 @@ class BackupSet:
         if self.remote_manifest_name:
             filelist.append(self.remote_manifest_name)
         filelist.extend(self.volume_name_dict.values())
-        return "[%s]" % ", ".join(filelist)
+        return u"[%s]" % u", ".join(map(util.ufn, filelist))
 
     def get_timestr(self):
         """
@@ -218,10 +233,10 @@ class BackupSet:
         # public key w/o secret key
         try:
             manifest_buffer = self.backend.get_data(self.remote_manifest_name)
-        except GPGError, message:
-            #TODO: We check for gpg v1 and v2 messages, should be an error code.
+        except GPGError as message:
+            # TODO: We check for gpg v1 and v2 messages, should be an error code
             if ("secret key not available" in message.args[0] or
-                "No secret key" in message.args[0]):
+                    "No secret key" in message.args[0]):
                 return None
             else:
                 raise
@@ -243,8 +258,7 @@ class BackupSet:
         assert self.info_set
         volume_num_list = self.volume_name_dict.keys()
         volume_num_list.sort()
-        volume_filenames = map(lambda x: self.volume_name_dict[x],
-                               volume_num_list)
+        volume_filenames = [self.volume_name_dict[x] for x in volume_num_list]
         if self.remote_manifest_name:
             # For convenience of implementation for restart support, we treat
             # local partial manifests as this set's remote manifest.  But
@@ -265,6 +279,9 @@ class BackupSet:
             return self.end_time
         assert 0, "Neither self.time nor self.end_time set"
 
+    def get_files_changed(self):
+        return self.files_changed
+
     def __len__(self):
         """
         Return the number of volumes in the set
@@ -285,7 +302,7 @@ class BackupChain:
         """
         self.backend = backend
         self.fullset = None
-        self.incset_list = [] # sorted list of BackupSets
+        self.incset_list = []  # sorted list of BackupSets
         self.start_time, self.end_time = None, None
 
     def set_full(self, fullset):
@@ -305,8 +322,8 @@ class BackupChain:
             self.incset_list.append(incset)
         else:
             if (self.incset_list
-                and incset.start_time == self.incset_list[-1].start_time
-                and incset.end_time > self.incset_list[-1]):
+                    and incset.start_time == self.incset_list[-1].start_time
+                    and incset.end_time > self.incset_list[-1]):
                 log.Info(_("Preferring Backupset over previous one!"))
                 self.incset_list[-1] = incset
             else:
@@ -325,7 +342,7 @@ class BackupChain:
         """
         Delete all sets in chain, in reverse order
         """
-        for i in range(len(self.incset_list)-1, -1, -1):
+        for i in range(len(self.incset_list) - 1, -1, -1):
             self.incset_list[i].delete()
         if self.fullset and not keep_full:
             self.fullset.delete()
@@ -334,7 +351,7 @@ class BackupChain:
         """
         Return a list of sets in chain earlier or equal to time
         """
-        older_incsets = filter(lambda s: s.end_time <= time, self.incset_list)
+        older_incsets = [s for s in self.incset_list if s.end_time <= time]
         return [self.fullset] + older_incsets
 
     def get_last(self):
@@ -358,7 +375,7 @@ class BackupChain:
         suitable for log messages.
         """
         return "[%s]-[%s]" % (dup_time.timetopretty(self.start_time),
-                      dup_time.timetopretty(self.end_time))
+                              dup_time.timetopretty(self.end_time))
 
     def to_log_info(self, prefix=''):
         """
@@ -388,7 +405,7 @@ class BackupChain:
              _("Chain start time: ") + dup_time.timetopretty(self.start_time),
              _("Chain end time: ") + dup_time.timetopretty(self.end_time),
              _("Number of contained backup sets: %d") %
-             (len(self.incset_list)+1,),
+             (len(self.incset_list) + 1,),
              _("Total number of contained volumes: %d") %
              (self.get_num_volumes(),),
              set_schema % (_("Type of backup set:"), _("Time:"), _("Num volumes:"))]
@@ -449,8 +466,8 @@ class SignatureChain:
             self.archive_dir, self.backend = location, None
         else:
             self.archive_dir, self.backend = None, location
-        self.fullsig = None # filename of full signature
-        self.inclist = [] # list of filenames of incremental signatures
+        self.fullsig = None  # filename of full signature
+        self.inclist = []  # list of filenames of incremental signatures
         self.start_time, self.end_time = None, None
 
     def __str__(self):
@@ -484,7 +501,7 @@ class SignatureChain:
         else:
             return False
 
-    def add_filename(self, filename, pr = None):
+    def add_filename(self, filename, pr=None):
         """
         Add new sig filename to current chain.  Return true if fits
         """
@@ -510,20 +527,20 @@ class SignatureChain:
             self.start_time, self.end_time = pr.time, pr.time
             return 1
 
-    def get_fileobjs(self, time = None):
+    def get_fileobjs(self, time=None):
         """
         Return ordered list of signature fileobjs opened for reading,
         optionally at a certain time
         """
         assert self.fullsig
-        if self.archive_dir: # local
+        if self.archive_dir:  # local
             def filename_to_fileobj(filename):
                 """Open filename in archive_dir, return filtered fileobj"""
                 sig_dp = path.DupPath(self.archive_dir.name, (filename,))
                 return sig_dp.filtered_open("rb")
         else:
             filename_to_fileobj = self.backend.get_fileobj_read
-        return map(filename_to_fileobj, self.get_filenames(time))
+        return [filename_to_fileobj(f) for f in self.get_filenames(time)]
 
     def delete(self, keep_full=False):
         """
@@ -531,7 +548,7 @@ class SignatureChain:
         """
         # Try to delete in opposite order, so something useful even if aborted
         if self.archive_dir:
-            for i in range(len(self.inclist)-1, -1, -1):
+            for i in range(len(self.inclist) - 1, -1, -1):
                 self.archive_dir.append(self.inclist[i]).delete()
             if not keep_full:
                 self.archive_dir.append(self.fullsig).delete()
@@ -543,7 +560,7 @@ class SignatureChain:
                 inclist_copy.append(self.fullsig)
             self.backend.delete(inclist_copy)
 
-    def get_filenames(self, time = None):
+    def get_filenames(self, time=None):
         """
         Return ordered list of filenames in set, up to a provided time
         """
@@ -611,42 +628,42 @@ class CollectionsStatus:
 
         return l
 
-    def __str__(self):
+    def __unicode__(self):
         """
         Return string summary of the collection
         """
         l = [_("Collection Status"),
-             "-----------------",
+             u"-----------------",
              _("Connecting with backend: %s") %
              (self.backend.__class__.__name__,),
-             _("Archive dir: %s") % (self.archive_dir.name,)]
+             _("Archive dir: %s") % (util.ufn(self.archive_dir.name),)]
 
         l.append("\n" +
-                 gettext.ngettext("Found %d secondary backup chain.",
-                                  "Found %d secondary backup chains.",
-                                  len(self.other_backup_chains))
+                 ngettext("Found %d secondary backup chain.",
+                          "Found %d secondary backup chains.",
+                          len(self.other_backup_chains))
                  % len(self.other_backup_chains))
         for i in range(len(self.other_backup_chains)):
             l.append(_("Secondary chain %d of %d:") %
-                     (i+1, len(self.other_backup_chains)))
-            l.append(str(self.other_backup_chains[i]))
+                     (i + 1, len(self.other_backup_chains)))
+            l.append(unicode(self.other_backup_chains[i]))
             l.append("")
 
         if self.matched_chain_pair:
             l.append("\n" + _("Found primary backup chain with matching "
                      "signature chain:"))
-            l.append(str(self.matched_chain_pair[1]))
+            l.append(unicode(self.matched_chain_pair[1]))
         else:
             l.append(_("No backup chains with active signatures found"))
 
         if self.orphaned_backup_sets or self.incomplete_backup_sets:
-            l.append(gettext.ngettext("Also found %d backup set not part of any chain,",
-                                      "Also found %d backup sets not part of any chain,",
-                                      len(self.orphaned_backup_sets))
+            l.append(ngettext("Also found %d backup set not part of any chain,",
+                              "Also found %d backup sets not part of any chain,",
+                              len(self.orphaned_backup_sets))
                      % (len(self.orphaned_backup_sets),))
-            l.append(gettext.ngettext("and %d incomplete backup set.",
-                                      "and %d incomplete backup sets.",
-                                      len(self.incomplete_backup_sets))
+            l.append(ngettext("and %d incomplete backup set.",
+                              "and %d incomplete backup sets.",
+                              len(self.incomplete_backup_sets))
                      % (len(self.incomplete_backup_sets),))
             # TRANSL: "cleanup" is a hard-coded command, so do not translate it
             l.append(_('These may be deleted by running duplicity with the '
@@ -654,9 +671,9 @@ class CollectionsStatus:
         else:
             l.append(_("No orphaned or incomplete backup sets found."))
 
-        return "\n".join(l)
+        return u"\n".join(l)
 
-    def set_values(self, sig_chain_warning = 1):
+    def set_values(self, sig_chain_warning=1):
         """
         Set values from archive_dir and backend.
 
@@ -668,16 +685,16 @@ class CollectionsStatus:
 
         # get remote filename list
         backend_filename_list = self.backend.list()
-        log.Debug(gettext.ngettext("%d file exists on backend",
-                                   "%d files exist on backend",
-                                   len(backend_filename_list)) %
+        log.Debug(ngettext("%d file exists on backend",
+                           "%d files exist on backend",
+                           len(backend_filename_list)) %
                   len(backend_filename_list))
 
         # get local filename list
         local_filename_list = self.archive_dir.listdir()
-        log.Debug(gettext.ngettext("%d file exists in cache",
-                                   "%d files exist in cache",
-                                   len(local_filename_list)) %
+        log.Debug(ngettext("%d file exists in cache",
+                           "%d files exist in cache",
+                           len(local_filename_list)) %
                   len(local_filename_list))
 
         # check for partial backups
@@ -689,17 +706,17 @@ class CollectionsStatus:
 
         # get various backup sets and chains
         (backup_chains, self.orphaned_backup_sets,
-                 self.incomplete_backup_sets) = \
-                 self.get_backup_chains(partials + backend_filename_list)
+         self.incomplete_backup_sets) = \
+            self.get_backup_chains(partials + backend_filename_list)
         backup_chains = self.get_sorted_chains(backup_chains)
         self.all_backup_chains = backup_chains
 
         assert len(backup_chains) == len(self.all_backup_chains), "get_sorted_chains() did something more than re-ordering"
 
         local_sig_chains, self.local_orphaned_sig_names = \
-                            self.get_signature_chains(True)
+            self.get_signature_chains(True)
         remote_sig_chains, self.remote_orphaned_sig_names = \
-                            self.get_signature_chains(False, filelist = backend_filename_list)
+            self.get_signature_chains(False, filelist=backend_filename_list)
         self.set_matched_chain_pair(local_sig_chains + remote_sig_chains,
                                     backup_chains)
         self.warn(sig_chain_warning)
@@ -720,7 +737,7 @@ class CollectionsStatus:
         self.matched_chain_pair = None
         if sig_chains and backup_chains:
             latest_backup_chain = backup_chains[-1]
-            for i in range(len(sig_chains)-1, -1, -1):
+            for i in range(len(sig_chains) - 1, -1, -1):
                 if sig_chains[i].end_time == latest_backup_chain.end_time:
                     pass
                 # See if the set before last matches:
@@ -735,7 +752,7 @@ class CollectionsStatus:
                     continue
 
                 # Found a matching pair:
-                if self.matched_chain_pair == None:
+                if self.matched_chain_pair is None:
                     self.matched_chain_pair = (sig_chains[i], latest_backup_chain)
 
                 break
@@ -750,21 +767,21 @@ class CollectionsStatus:
         assert self.values_set
 
         if self.local_orphaned_sig_names:
-            log.Warn(gettext.ngettext("Warning, found the following local orphaned "
-                                      "signature file:",
-                                      "Warning, found the following local orphaned "
-                                      "signature files:",
-                                      len(self.local_orphaned_sig_names))
-                     + "\n" + "\n".join(self.local_orphaned_sig_names),
+            log.Warn(ngettext("Warning, found the following local orphaned "
+                              "signature file:",
+                              "Warning, found the following local orphaned "
+                              "signature files:",
+                              len(self.local_orphaned_sig_names))
+                     + u"\n" + u"\n".join(map(util.ufn, self.local_orphaned_sig_names)),
                      log.WarningCode.orphaned_sig)
 
         if self.remote_orphaned_sig_names:
-            log.Warn(gettext.ngettext("Warning, found the following remote orphaned "
-                                      "signature file:",
-                                      "Warning, found the following remote orphaned "
-                                      "signature files:",
-                                      len(self.remote_orphaned_sig_names))
-                     + "\n" + "\n".join(self.remote_orphaned_sig_names),
+            log.Warn(ngettext("Warning, found the following remote orphaned "
+                              "signature file:",
+                              "Warning, found the following remote orphaned "
+                              "signature files:",
+                              len(self.remote_orphaned_sig_names))
+                     + u"\n" + u"\n".join(map(util.ufn, self.remote_orphaned_sig_names)),
                      log.WarningCode.orphaned_sig)
 
         if self.all_sig_chains and sig_chain_warning and not self.matched_chain_pair:
@@ -776,13 +793,12 @@ class CollectionsStatus:
                        "from aborted session"), log.WarningCode.incomplete_backup)
 
         if self.orphaned_backup_sets:
-            log.Warn(gettext.ngettext("Warning, found the following orphaned "
-                                      "backup file:",
-                                      "Warning, found the following orphaned "
-                                      "backup files:",
-                                      len(self.orphaned_backup_sets))
-                     + "\n" + "\n".join(map(lambda x: str(x),
-                                            self.orphaned_backup_sets)),
+            log.Warn(ngettext("Warning, found the following orphaned "
+                              "backup file:",
+                              "Warning, found the following orphaned "
+                              "backup files:",
+                              len(self.orphaned_backup_sets))
+                     + u"\n" + u"\n".join(map(unicode, self.orphaned_backup_sets)),
                      log.WarningCode.orphaned_backup)
 
     def get_backup_chains(self, filename_list):
@@ -795,28 +811,32 @@ class CollectionsStatus:
         missing files.
         """
         log.Debug(_("Extracting backup chains from list of files: %s")
-                  % filename_list)
+                  % [util.ufn(f) for f in filename_list])
         # First put filenames in set form
         sets = []
+
         def add_to_sets(filename):
             """
             Try adding filename to existing sets, or make new one
             """
             for set in sets:
                 if set.add_filename(filename):
-                    log.Debug(_("File %s is part of known set") % (filename,))
+                    log.Debug(_("File %s is part of known set") % (util.ufn(filename),))
                     break
             else:
-                log.Debug(_("File %s is not part of a known set; creating new set") % (filename,))
+                log.Debug(_("File %s is not part of a known set; creating new set") % (util.ufn(filename),))
                 new_set = BackupSet(self.backend)
                 if new_set.add_filename(filename):
                     sets.append(new_set)
                 else:
-                    log.Debug(_("Ignoring file (rejected by backup set) '%s'") % filename)
-        map(add_to_sets, filename_list)
+                    log.Debug(_("Ignoring file (rejected by backup set) '%s'") % util.ufn(filename))
+
+        for f in filename_list:
+            add_to_sets(f)
         sets, incomplete_sets = self.get_sorted_sets(sets)
 
         chains, orphaned_sets = [], []
+
         def add_to_chains(set):
             """
             Try adding set to existing chains, or make new one
@@ -831,12 +851,13 @@ class CollectionsStatus:
                 for chain in chains:
                     if chain.add_inc(set):
                         log.Debug(_("Added set %s to pre-existing chain %s") % (set.get_timestr(),
-                                                     chain.short_desc()))
+                                                                                chain.short_desc()))
                         break
                 else:
                     log.Debug(_("Found orphaned set %s") % (set.get_timestr(),))
                     orphaned_sets.append(set)
-        map(add_to_chains, sets)
+        for s in sets:
+            add_to_chains(s)
         return (chains, orphaned_sets, incomplete_sets)
 
     def get_sorted_sets(self, set_list):
@@ -852,9 +873,9 @@ class CollectionsStatus:
             else:
                 time_set_pairs.append((set.end_time, set))
         time_set_pairs.sort()
-        return (map(lambda p: p[1], time_set_pairs), incomplete_sets)
+        return ([p[1] for p in time_set_pairs], incomplete_sets)
 
-    def get_signature_chains(self, local, filelist = None):
+    def get_signature_chains(self, local, filelist=None):
         """
         Find chains in archive_dir (if local is true) or backend
 
@@ -913,7 +934,7 @@ class CollectionsStatus:
         # Build dictionary from end_times to lists of corresponding chains
         endtime_chain_dict = {}
         for chain in chain_list:
-            if endtime_chain_dict.has_key(chain.end_time):
+            if chain.end_time in endtime_chain_dict:
                 endtime_chain_dict[chain.end_time].append(chain)
             else:
                 endtime_chain_dict[chain.end_time] = [chain]
@@ -928,10 +949,10 @@ class CollectionsStatus:
                 sorted_chain_list.append(chain_list[0])
             else:
                 assert len(chain_list) == 2
-                if chain_list[0].backend: # is remote, goes first
+                if chain_list[0].backend:  # is remote, goes first
                     sorted_chain_list.append(chain_list[0])
                     sorted_chain_list.append(chain_list[1])
-                else: # is local, goes second
+                else:  # is local, goes second
                     sorted_chain_list.append(chain_list[1])
                     sorted_chain_list.append(chain_list[0])
 
@@ -948,19 +969,18 @@ class CollectionsStatus:
         if not self.all_backup_chains:
             raise CollectionsError("No backup chains found")
 
-        covering_chains = filter(lambda c: c.start_time <= time <= c.end_time,
-                                 self.all_backup_chains)
+        covering_chains = [c for c in self.all_backup_chains
+                           if c.start_time <= time <= c.end_time]
         if len(covering_chains) > 1:
             raise CollectionsError("Two chains cover the given time")
         elif len(covering_chains) == 1:
             return covering_chains[0]
 
-        old_chains = filter(lambda c: c.end_time < time,
-                            self.all_backup_chains)
+        old_chains = [c for c in self.all_backup_chains if c.end_time < time]
         if old_chains:
             return old_chains[-1]
         else:
-            return self.all_backup_chains[0] # no chains are old enough
+            return self.all_backup_chains[0]  # no chains are old enough
 
     def get_signature_chain_at_time(self, time):
         """
@@ -973,13 +993,12 @@ class CollectionsStatus:
         if not self.all_sig_chains:
             raise CollectionsError("No signature chains found")
 
-        covering_chains = filter(lambda c: c.start_time <= time <= c.end_time,
-                                 self.all_sig_chains)
+        covering_chains = [c for c in self.all_sig_chains
+                           if c.start_time <= time <= c.end_time]
         if covering_chains:
-            return covering_chains[-1] # prefer local if multiple sig chains
+            return covering_chains[-1]  # prefer local if multiple sig chains
 
-        old_chains = filter(lambda c: c.end_time < time,
-                            self.all_sig_chains)
+        old_chains = [c for c in self.all_sig_chains if c.end_time < time]
         if old_chains:
             return old_chains[-1]
         else:
@@ -1007,7 +1026,7 @@ class CollectionsStatus:
                 matched_sig_chain = self.matched_chain_pair[0]
                 for sig_chain in self.all_sig_chains:
                     if (sig_chain.start_time == matched_sig_chain.start_time and
-                        sig_chain.end_time == matched_sig_chain.end_time):
+                            sig_chain.end_time == matched_sig_chain.end_time):
                         old_sig_chains.remove(sig_chain)
             ext_containers += old_sig_chains
         for set_or_chain in ext_containers:
@@ -1021,9 +1040,9 @@ class CollectionsStatus:
 
     def sort_sets(self, setlist):
         """Return new list containing same elems of setlist, sorted by time"""
-        pairs = map(lambda s: (s.get_time(), s), setlist)
+        pairs = [(s.get_time(), s) for s in setlist]
         pairs.sort()
-        return map(lambda p: p[1], pairs)
+        return [p[1] for p in pairs]
 
     def get_chains_older_than(self, t):
         """
@@ -1038,11 +1057,11 @@ class CollectionsStatus:
         assert self.values_set
         old_chains = []
         for chain in self.all_backup_chains:
-            if chain.end_time < t and (
-                not self.matched_chain_pair or
-                chain is not self.matched_chain_pair[1]):
-                # don't delete the active (matched) chain
-                old_chains.append(chain)
+            if (chain.end_time < t and
+                (not self.matched_chain_pair or
+                 chain is not self.matched_chain_pair[1])):
+                    # don't delete the active (matched) chain
+                    old_chains.append(chain)
         return old_chains
 
     def get_signature_chains_older_than(self, t):
@@ -1058,11 +1077,11 @@ class CollectionsStatus:
         assert self.values_set
         old_chains = []
         for chain in self.all_sig_chains:
-            if chain.end_time < t and (
-                not self.matched_chain_pair or
-                chain is not self.matched_chain_pair[0]):
-                # don't delete the active (matched) chain
-                old_chains.append(chain)
+            if (chain.end_time < t and
+                (not self.matched_chain_pair or
+                 chain is not self.matched_chain_pair[0])):
+                    # don't delete the active (matched) chain
+                    old_chains.append(chain)
         return old_chains
 
     def get_last_full_backup_time(self):
@@ -1090,7 +1109,7 @@ class CollectionsStatus:
         """
         return self.get_nth_last_backup_chain(1)
 
-    def get_nth_last_backup_chain(self,n):
+    def get_nth_last_backup_chain(self, n):
         """
         Return the nth-to-last full backup of the collection,
         or None if there is less than n backup chains.
@@ -1144,3 +1163,53 @@ class CollectionsStatus:
             old_sets = filter(lambda s: s.get_time() < t, chain.get_all_sets())
             result_sets.extend(old_sets)
         return self.sort_sets(result_sets)
+
+    def get_file_changed_record(self, filepath):
+        """
+        Returns time line of specified file changed
+        """
+        # quick fix to spaces in filepath
+        modified_filepath = ""
+        if " " in filepath:
+            modified_filepath = '"' + filepath.replace(" ", r"\x20") + '"'
+
+        if not self.matched_chain_pair:
+            return ""
+
+        all_backup_set = self.matched_chain_pair[1].get_all_sets()
+        specified_file_backup_set = []
+        specified_file_backup_type = []
+
+        for bs in all_backup_set:
+            filelist = [fileinfo[1] for fileinfo in bs.get_files_changed()]
+            if modified_filepath in filelist:
+                specified_file_backup_set.append(bs)
+                index = filelist.index(modified_filepath)
+                specified_file_backup_type.append(bs.get_files_changed()[index][0])
+
+        return FileChangedStatus(filepath, list(zip(specified_file_backup_type, specified_file_backup_set)))
+
+
+class FileChangedStatus:
+    def __init__(self, filepath, fileinfo_list):
+        self.filepath = filepath
+        self.fileinfo_list = fileinfo_list
+
+    def __unicode__(self):
+        set_schema = "%20s   %30s  %20s"
+        l = ["-------------------------",
+             _("File: %s") % (self.filepath),
+             _("Total number of backup: %d") % len(self.fileinfo_list),
+             set_schema % (_("Type of backup set:"), _("Time:"), _("Type of file change:"))]
+
+        for s in self.fileinfo_list:
+            backup_type = s[0]
+            backup_set = s[1]
+            if backup_set.time:
+                type = _("Full")
+            else:
+                type = _("Incremental")
+            l.append(set_schema % (type, dup_time.timetopretty(backup_set.get_time()), backup_type.title()))
+
+        l.append("-------------------------")
+        return "\n".join(l)
